'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _omniFetch = require('omni-fetch');

var _omniFetch2 = _interopRequireDefault(_omniFetch);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _async = require('async.queue');

var _async2 = _interopRequireDefault(_async);

var _isStream = require('is-stream');

var _isStream2 = _interopRequireDefault(_isStream);

var _write = require('write');

var _write2 = _interopRequireDefault(_write);

var _inistArk = require('inist-ark');

var _inistArk2 = _interopRequireDefault(_inistArk);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ark = new _inistArk2.default();
var worker = function worker(options) {
    return function (_ref, done) {
        var source = _ref.source,
            target = _ref.target;
        return (0, _omniFetch2.default)(source, options).then(function (response) {
            if (!(0, _isStream2.default)(response.body)) {
                return done(new Error('Unexpected response'));
            }
            response.body.pipe(_write2.default.stream(target)).on('end', function (err) {
                return done(err);
            }).on('close', function () {
                return done(target);
            });
        });
    };
};

function ISTEXSave(data, feed) {
    if (this.isLast()) {
        return feed.close();
    }
    var directory = this.getParam('directory', process.cwd());
    var typology = this.getParam('typology', 'fulltext');
    var format = this.getParam('format', 'pdf');
    var sid = this.getParam('sid', 'ezs-istex');
    var token = this.getParam('token');
    var headers = {};
    var location = {
        protocol: 'https:',
        host: 'api.istex.fr'
    };
    if (token) {
        headers.Authorization = 'Bearer ' + token;
    }
    if (!data.hits && !data.arkIstex) {
        throw new Error('[ISTEXFetch] or [ISTEXSearch] should be defined before this statement.');
    }
    if (!format && !typology) {
        throw new Error('typology= & format= must be defined as parameter.');
    }

    var identifiers = data.hits ? data.hits.map(function (hit) {
        return hit.arkIstex;
    }) : [data.arkIstex];
    var files = identifiers.map(function (arkIstex) {
        var _ark$parse = ark.parse(arkIstex),
            arkName = _ark$parse.name;

        var pathname = arkIstex + '/' + typology + '.' + format;
        var filename = arkName + '-' + typology + '.' + format;
        var urlObj = (0, _extends3.default)({}, location, {
            pathname: pathname,
            query: {
                sid: sid
            }
        });
        var cmdObj = {
            source: _url2.default.format(urlObj),
            target: _path2.default.resolve(directory, filename)
        };
        return cmdObj;
    });
    var o = {
        headers: headers,
        sid: sid
    };
    var q = (0, _async2.default)(worker(o), _os2.default.cpus().length);
    q.drain = function () {
        return feed.end();
    };
    files.forEach(function (file) {
        return q.push(file, function (filename) {
            return feed.write(filename);
        });
    });
}

exports.default = {
    ISTEXSave: ISTEXSave
};