'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _dotProp = require('dot-prop');

var _dotProp2 = _interopRequireDefault(_dotProp);

var _omniFetch = require('omni-fetch');

var _omniFetch2 = _interopRequireDefault(_omniFetch);

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ISTEXSearch(data, feed) {
    if (this.isLast()) {
        return feed.close();
    }
    var source = this.getParam('source', 'query');
    var target = this.getParam('target');
    var query = this.getParam('query', _dotProp2.default.get(data, source, data));
    var sid = this.getParam('sid', 'ezs-istex');
    var maxPage = Number(this.getParam('maxPage'));
    var size = Number(this.getParam('size', 2000));
    var scroll = this.getParam('duration', '30s');
    var field = this.getParam('field', ['doi']);
    var fields = Array.isArray(field) ? field : [field];
    var output = 'arkIstex,' + fields.map(function (e) {
        return (/\w+/.exec(e)[0]
        );
    }).join();
    var location = {
        protocol: 'https:',
        host: 'api.istex.fr',
        pathname: '/document/'
    };
    var parameters = {
        q: query,
        scroll: scroll,
        output: output,
        size: size,
        sid: sid
    };
    var urlObj = (0, _extends3.default)({}, location, {
        search: _qs2.default.stringify(parameters)
    });
    var urlStr = _url2.default.format(urlObj);
    (0, _omniFetch2.default)(urlStr).then(function (response) {
        return response.json();
    }).then(function (json) {
        if (json.total === 0) {
            return feed.send(new Error('No result.'));
        }
        if (json._error) {
            return feed.send(new Error(json._error));
        }
        if (json.total === undefined) {
            return feed.send(new Error('Unexpected response.'));
        }
        // first Page
        feed.write((0, _utils.newValue)(json, target, data));
        var scrollId = json.scrollId;

        var scrollUrl = _url2.default.format((0, _extends3.default)({}, location, {
            search: _qs2.default.stringify((0, _extends3.default)({}, parameters, {
                scrollId: scrollId
            }))
        }));
        // all other pages
        var pages = Array(Math.ceil(json.total / size) - 1).map(function () {
            return scrollUrl;
        }).fill(scrollUrl);
        if (maxPage) {
            pages.slice(0, maxPage - 1).forEach(function (pageURL) {
                return feed.write((0, _utils.newValue)(pageURL, target, data));
            });
        } else {
            pages.forEach(function (pageURL) {
                return feed.write((0, _utils.newValue)(pageURL, target, data));
            });
        }
        return feed.end();
    }).catch(function (err) {
        err.url = urlStr;
        feed.send(err);
    });
}

exports.default = {
    ISTEXSearch: ISTEXSearch
};