'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var ISTEX = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(data, feed) {
        var maxPage, size, sid, scroll, field, query, queries, identifier, identifiers, options;
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        if (!this.isLast()) {
                            _context.next = 2;
                            break;
                        }

                        return _context.abrupt('return', feed.close());

                    case 2:
                        maxPage = this.getParam('maxPage');
                        size = this.getParam('size');
                        sid = this.getParam('sid');
                        scroll = this.getParam('duration');
                        field = this.getParam('field');
                        query = this.getParam('query', data.query || []);
                        queries = Array.isArray(query) ? query : [query];
                        identifier = this.getParam('id', data.id || []);
                        identifiers = Array.isArray(identifier) ? identifier : [identifier];
                        options = {
                            maxPage: maxPage,
                            size: size,
                            scroll: scroll,
                            field: field,
                            sid: sid
                        };
                        _context.next = 14;
                        return getAndWriteQueries(queries, options, feed);

                    case 14:
                        _context.next = 16;
                        return getAndWriteIdentifiers(identifiers, options, feed);

                    case 16:
                        feed.end();

                    case 17:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this);
    }));

    return function ISTEX(_x, _x2) {
        return _ref.apply(this, arguments);
    };
}();

var _ezs = require('ezs');

var _ezs2 = _interopRequireDefault(_ezs);

var _stream = require('stream');

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _async = require('async.queue');

var _async2 = _interopRequireDefault(_async);

var _utils = require('./utils');

var _istexSearch = require('./istex-search');

var _istexSearch2 = _interopRequireDefault(_istexSearch);

var _istexScroll = require('./istex-scroll');

var _istexScroll2 = _interopRequireDefault(_istexScroll);

var _istexResult = require('./istex-result');

var _istexResult2 = _interopRequireDefault(_istexResult);

var _istexFetch = require('./istex-fetch');

var _istexFetch2 = _interopRequireDefault(_istexFetch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var worker = function worker(stream) {
    return function (data, done) {
        (0, _utils.writeTo)(stream, data, function () {
            return done();
        });
    };
};

var getAndWriteQueries = function getAndWriteQueries(data, options, feed) {
    return new Promise(function (resolve, reject) {
        if (!Array.isArray(data)) {
            return reject(new Error('unexpected data. Should be an array.'));
        }
        if (data.length === 0) {
            return resolve();
        }
        var inputQuery = new _stream.PassThrough({
            writableObjectMode: true,
            readableObjectMode: true
        });
        var q = (0, _async2.default)(worker(inputQuery), _os2.default.cpus().length);
        q.drain = function () {
            return inputQuery.end();
        };
        data.forEach(function (query) {
            return q.push({ query: query });
        });

        inputQuery.pipe((0, _ezs2.default)(_istexSearch2.default, options)).pipe((0, _ezs2.default)(_istexScroll2.default)).pipe((0, _ezs2.default)(_istexResult2.default)).on('data', function (chunk) {
            feed.write(chunk);
        }).on('end', function () {
            resolve();
        }).on('error', function (e) {
            reject(e);
        });
    });
};

var getAndWriteIdentifiers = function getAndWriteIdentifiers(data, options, feed) {
    return new Promise(function (resolve, reject) {
        if (!Array.isArray(data)) {
            return reject(new Error('unexpected data. Should be an array.'));
        }
        if (data.length === 0) {
            return resolve();
        }
        var input = new _stream.PassThrough({
            writableObjectMode: true,
            readableObjectMode: true
        });
        var q = (0, _async2.default)(worker(input), _os2.default.cpus().length);
        q.drain = function () {
            return input.end();
        };
        data.forEach(function (id) {
            return q.push({ id: id });
        });

        input.pipe((0, _ezs2.default)(_istexFetch2.default, options)).on('data', function (chunk) {
            feed.write(chunk);
        }).on('end', function () {
            resolve();
        }).on('error', function (e) {
            reject(e);
        });
    });
};

exports.default = {
    ISTEX: ISTEX
};